# @HEADER
# *****************************************************************************
#            TriBITS: Tribal Build, Integrate, and Test System
#
# Copyright 2013-2016 NTESS and the TriBITS contributors.
# SPDX-License-Identifier: BSD-3-Clause
# *****************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/Phalanx clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "Phalanx requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/Phalanx build
## ---------------------------------------------------------------------------

set(Phalanx_CXX_COMPILER "/usr/bin/c++")

set(Phalanx_C_COMPILER "/usr/bin/cc")

set(Phalanx_Fortran_COMPILER "/usr/local/bin/gfortran13")
# Deprecated!
set(Phalanx_FORTRAN_COMPILER "/usr/local/bin/gfortran13") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/Phalanx build
## ---------------------------------------------------------------------------

## Give the build type
set(Phalanx_CMAKE_BUILD_TYPE "Release")

## Set compiler flags, including those determined by build type
set(Phalanx_CXX_FLAGS [[ ]])

set(Phalanx_C_FLAGS [[-O2 -pipe  -fstack-protector-strong -fno-strict-aliasing ]])

set(Phalanx_Fortran_FLAGS [[ ]])
# Deprecated
set(Phalanx_FORTRAN_FLAGS [[ ]])

## Extra link flags (e.g., specification of fortran libraries)
set(Phalanx_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(Phalanx_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/local/lib")
set(Phalanx_BUILD_SHARED_LIBS "ON")

set(Phalanx_LINKER /usr/bin/ld.lld)
set(Phalanx_AR /usr/bin/llvm-ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(Phalanx_INSTALL_DIR "/usr/local")

## List of package libraries
set(Phalanx_LIBRARIES Phalanx::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(Phalanx_MPI_LIBRARIES "")
set(Phalanx_MPI_LIBRARY_DIRS "")
set(Phalanx_MPI_INCLUDE_DIRS "")
set(Phalanx_MPI_EXEC "")
set(Phalanx_MPI_EXEC_MAX_NUMPROCS "")
set(Phalanx_MPI_EXEC_NUMPROCS_FLAG "")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(Phalanx_ENABLE_TeuchosCore ON)
set(Phalanx_ENABLE_TeuchosParameterList ON)
set(Phalanx_ENABLE_TeuchosComm ON)
set(Phalanx_ENABLE_Kokkos ON)
set(Phalanx_ENABLE_Sacado ON)

# Exported cache variables
set(Phalanx_ALLOW_MULTIPLE_EVALUATORS_FOR_SAME_FIELD "ON")
set(PHX_ALLOW_MULTIPLE_EVALUATORS_FOR_SAME_FIELD "ON")
set(Phalanx_EXPLICIT_TEMPLATE_INSTANTIATION "OFF")
set(PHX_ETI "OFF")
set(Phalanx_ENABLE_DEBUG "OFF")
set(PHX_DEBUG "OFF")
set(Phalanx_ENABLE_TEUCHOS_TIME_MONITOR "ON")
set(PHX_TEUCHOS_TIME_MONITOR "ON")
set(Phalanx_ENABLE_DEVICE_DAG "OFF")
set(PHX_ENABLE_DEVICE_DAG "OFF")
set(Phalanx_ENABLE_KOKKOS_AMT "OFF")
set(PHX_ENABLE_KOKKOS_AMT "OFF")

# Include configuration of dependent packages
if (NOT TARGET TeuchosCore::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../TeuchosCore/TeuchosCoreConfig.cmake")
endif()
if (NOT TARGET TeuchosParameterList::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../TeuchosParameterList/TeuchosParameterListConfig.cmake")
endif()
if (NOT TARGET TeuchosComm::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../TeuchosComm/TeuchosCommConfig.cmake")
endif()
if (NOT TARGET Kokkos::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Kokkos/KokkosConfig.cmake")
endif()
if (NOT TARGET Sacado::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Sacado/SacadoConfig.cmake")
endif()

# Import Phalanx targets
include("${CMAKE_CURRENT_LIST_DIR}/PhalanxTargets.cmake")

# Standard TriBITS-compliant external package variables
set(Phalanx_IS_TRIBITS_COMPLIANT TRUE)
set(Phalanx_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(Phalanx_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(Phalanx_EXPORTED_PACKAGE_LIBS_NAMES "phalanx")

foreach(libname IN LISTS Phalanx_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE Phalanx::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'Phalanx::${libname}', or better yet,"
      " 'Phalanx::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'Phalanx'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'Phalanx_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
