# @HEADER
# *****************************************************************************
#            TriBITS: Tribal Build, Integrate, and Test System
#
# Copyright 2013-2016 NTESS and the TriBITS contributors.
# SPDX-License-Identifier: BSD-3-Clause
# *****************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/Piro clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "Piro requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/Piro build
## ---------------------------------------------------------------------------

set(Piro_CXX_COMPILER "/usr/bin/c++")

set(Piro_C_COMPILER "/usr/bin/cc")

set(Piro_Fortran_COMPILER "/usr/local/bin/gfortran13")
# Deprecated!
set(Piro_FORTRAN_COMPILER "/usr/local/bin/gfortran13") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/Piro build
## ---------------------------------------------------------------------------

## Give the build type
set(Piro_CMAKE_BUILD_TYPE "Release")

## Set compiler flags, including those determined by build type
set(Piro_CXX_FLAGS [[ ]])

set(Piro_C_FLAGS [[-O2 -pipe  -fstack-protector-strong -fno-strict-aliasing ]])

set(Piro_Fortran_FLAGS [[ ]])
# Deprecated
set(Piro_FORTRAN_FLAGS [[ ]])

## Extra link flags (e.g., specification of fortran libraries)
set(Piro_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(Piro_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/local/lib")
set(Piro_BUILD_SHARED_LIBS "ON")

set(Piro_LINKER /usr/bin/ld.lld)
set(Piro_AR /usr/bin/llvm-ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(Piro_INSTALL_DIR "/usr/local")

## List of package libraries
set(Piro_LIBRARIES Piro::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(Piro_MPI_LIBRARIES "")
set(Piro_MPI_LIBRARY_DIRS "")
set(Piro_MPI_INCLUDE_DIRS "")
set(Piro_MPI_EXEC "")
set(Piro_MPI_EXEC_MAX_NUMPROCS "")
set(Piro_MPI_EXEC_NUMPROCS_FLAG "")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(Piro_ENABLE_Teuchos ON)
set(Piro_ENABLE_Stratimikos ON)
set(Piro_ENABLE_ThyraCore ON)
set(Piro_ENABLE_Tpetra ON)
set(Piro_ENABLE_Teko ON)
set(Piro_ENABLE_NOX ON)
set(Piro_ENABLE_Tempus ON)
set(Piro_ENABLE_ROL ON)
set(Piro_ENABLE_Ifpack2 ON)
set(Piro_ENABLE_MueLu ON)
set(Piro_ENABLE_ThyraEpetraAdapters OFF)
set(Piro_ENABLE_ThyraEpetraExtAdapters OFF)
set(Piro_ENABLE_Epetra ON)
set(Piro_ENABLE_EpetraExt ON)

# Exported cache variables

# Include configuration of dependent packages
if (NOT TARGET Teuchos::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Teuchos/TeuchosConfig.cmake")
endif()
if (NOT TARGET Stratimikos::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Stratimikos/StratimikosConfig.cmake")
endif()
if (NOT TARGET ThyraCore::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../ThyraCore/ThyraCoreConfig.cmake")
endif()
if (NOT TARGET Tpetra::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Tpetra/TpetraConfig.cmake")
endif()
if (NOT TARGET Teko::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Teko/TekoConfig.cmake")
endif()
if (NOT TARGET NOX::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../NOX/NOXConfig.cmake")
endif()
if (NOT TARGET Tempus::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Tempus/TempusConfig.cmake")
endif()
if (NOT TARGET ROL::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../ROL/ROLConfig.cmake")
endif()
if (NOT TARGET Ifpack2::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Ifpack2/Ifpack2Config.cmake")
endif()
if (NOT TARGET MueLu::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../MueLu/MueLuConfig.cmake")
endif()
if (NOT TARGET Epetra::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Epetra/EpetraConfig.cmake")
endif()
if (NOT TARGET EpetraExt::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../EpetraExt/EpetraExtConfig.cmake")
endif()

# Import Piro targets
include("${CMAKE_CURRENT_LIST_DIR}/PiroTargets.cmake")

# Standard TriBITS-compliant external package variables
set(Piro_IS_TRIBITS_COMPLIANT TRUE)
set(Piro_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(Piro_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(Piro_EXPORTED_PACKAGE_LIBS_NAMES "piro")

foreach(libname IN LISTS Piro_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE Piro::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'Piro::${libname}', or better yet,"
      " 'Piro::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'Piro'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'Piro_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
